indexing

   description: "Base class for simple CGI programs. Outputs XHTML."

   usage: "Never write to stdout, use `puts' or `add_data' instead."

   known_bugs:
   "1. Does not handle inputs with same name (checkboxes) correctly."

   todo:
   "1. should use binary search allowed_new_line_tag"

   author: "Berend de Boer"
   date: "$Date: 2001/12/27 $"
   revision: "$Revision: #1 $"

deferred class

   EPX_CGI

inherit

   STDC_CURRENT_PROCESS

   XHTML_GENERATOR
      redefine
         make,
         extend
      end


feature -- creation

   make is
      do
         precursor
--           debug
--              dump_input
--              exit (1)
--           end
         execute
--        rescue
--           handle_fatal_error
      end


feature -- the output routine

   execute is
         -- to be implemented by child
      deferred
      end


feature -- when script fails

   handle_fatal_error is
         -- your last change to emit something when the script has
         -- failed unexpectedly
      do
         stdout.puts ("A fatal error prevents this program from continuing.%N")
         exit (EXIT_FAILURE)
      end


feature -- debug support

   dump_input is
         -- write cgi input to /tmp/cgi_input
      local
         file: STDC_BINARY_FILE
         clen: INTEGER
         i: INTEGER
      do
         create file.create_write ("/tmp/cgi_input")
         if request_method.is_equal ("GET") then
            file.puts (query_string)
         elseif request_method.is_equal ("POST") then
            from
               i := 0
               clen := content_length
            until
               i >= clen
            loop
               stdin.get_character
               file.put_character (stdin.last_byte)
               i := i + 1
            end
         end
         file.close
      end


feature -- overrule some xml stuff

   extend (stuff: STRING) is
         -- output cgi code, never write to `stdout' directly!
      do
         if stuff /= Void then
            stdout.puts (stuff)
         end
         precursor (stuff)
      end


feature -- standard variables

   auth_type: STRING is
         -- type of authentication used
      do
         Result := env_auth_type.value
      end

   content_type: STRING is
         -- MIME type of data when invoked with POST method
      do
         Result := env_content_type.value
      end

   content_length: INTEGER is
         -- length, in bytes, of data when invoked with POST method
      do
         if env_content_length.value.is_equal ("") then
            Result := 0
         else
            Result := env_content_length.value.to_integer
         end
      end

   gateway_interface: STRING is
         -- name and version of the gateway, for example CGI/1.1
      do
         Result := env_gateway_interface.value
      end

   http_accept: STRING is
         -- contents of the Accept header line sent by the client
      do
         Result := env_http_accept.value
      end

   http_referer: STRING is
         -- contents of the Referer header line
      do
         Result := env_http_referer.value
      end

   http_user_agent: STRING is
         -- name of the client program that is making the request
      do
         Result := env_http_user_agent.value
      end

   path_info: STRING is
         -- extra path information as it was passed to the server in
         -- the query URL
      do
         Result := env_path_info.value
      end

   path_translated: STRING is
         -- extra path information translated to a final, usable
         -- form. The Web document root is prepended to the query
         -- path, and any other path translations are executed.
      do
         Result := env_path_translated.value
      end

   query_string: STRING is
         -- the input when invoked with the GET method
      do
         Result := env_query_string.value
      end

   remote_addr, remote_address: STRING is
         -- IP address of the client that made the request
      do
         Result := env_remote_addr.value
      end

   remote_host: STRING is
         -- name of the remote computer that made the request
      do
         Result := env_remote_host.value
      end

   remote_ident: STRING is
         -- user name as given by the ident protocol
      do
         Result := env_remote_ident.value
      end

   remote_user: STRING is
         -- name of the remote user that made the request
      do
         Result := env_remote_user.value
      end

   request_method: STRING is
         -- name of the method used to invoke the CGI
         -- application. Valid values are GET and POST
      do
         Result := env_request_method.value
      end

   script_name: STRING is
         -- name of script that was invoked
      do
         Result := env_script_name.value
      end

   server_name: STRING is
         -- domain name of the computer that is running the server software
      do
         Result := env_server_name.value
      end

   server_port: INTEGER is
         -- TCP port number on which the server that invoked the CGI
         -- application is operating
      local
         s: STRING
      do
         s := env_server_port.value
         if s.is_empty then
            Result := 0
         else
            Result := s.to_integer
         end
      end

   server_protocol: STRING is
         -- name of the protocol that the server is using and the
         -- version of that protocol. The protocol name and version
         -- are separated by a forward slash with no spaces, for
         -- instance HTTP/1.0
      do
         Result := env_server_protocol.value
      end

   server_software: STRING is
         -- name of the server that is handling the request
      do
         Result := env_server_software.value
      end


feature {NONE} -- environment variables

   env_auth_type: STDC_ENV_VAR is
      once
         create Result.make ("AUTH_TYPE")
      end

   env_content_length: STDC_ENV_VAR is
      once
         create Result.make ("CONTENT_LENGTH")
      end

   env_content_type: STDC_ENV_VAR is
      once
         create Result.make ("CONTENT_TYPE")
      end

   env_gateway_interface: STDC_ENV_VAR is
      once
         create Result.make ("GATEWAY_INTERFACE")
      end

   env_http_accept: STDC_ENV_VAR is
      once
         create Result.make ("HTTP_ACCEPT")
      end

   env_http_referer: STDC_ENV_VAR is
      once
         create Result.make ("HTTP_REFERER")
      end

   env_http_user_agent: STDC_ENV_VAR is
      once
         create Result.make ("HTTP_USER_AGENT")
      end

   env_path_info: STDC_ENV_VAR is
      once
         create Result.make ("PATH_INFO")
      end

   env_path_translated: STDC_ENV_VAR is
      once
         create Result.make ("PATH_TRANSLATED")
      end

   env_query_string: STDC_ENV_VAR is
      once
         create Result.make ("QUERY_STRING")
      end

   env_remote_addr: STDC_ENV_VAR is
      once
         create Result.make ("REMOTE_ADDR")
      end

   env_remote_host: STDC_ENV_VAR is
      once
         create Result.make ("REMOTE_HOST")
      end

   env_remote_user: STDC_ENV_VAR is
      once
         create Result.make ("REMOTE_USER")
      end

   env_remote_ident: STDC_ENV_VAR is
      once
         create Result.make ("REMOTE_IDENT")
      end

   env_request_method: STDC_ENV_VAR is
      once
         create Result.make ("REQUEST_METHOD")
      end

   env_script_name: STDC_ENV_VAR is
      once
         create Result.make ("SCRIPT_NAME")
      end

   env_server_name: STDC_ENV_VAR is
      once
         create Result.make ("SERVER_NAME")
      end

   env_server_port: STDC_ENV_VAR is
      once
         create Result.make ("SERVER_PORT")
      end

   env_server_protocol: STDC_ENV_VAR is
      once
         create Result.make ("SERVER_PROTOCOL")
      end

   env_server_software: STDC_ENV_VAR is
      once
         create Result.make ("SERVER_SOFTWARE")
      end


feature -- standard cgi header

   content_text is
      obsolete "content_text is obsolete, use context_text_html"
      do
         content_text_html
      end

   content_text_html is
      do
         extend ("Content-Type: text/html")
         new_line
         new_line
         stdout.flush
      end

   content_text_plain is
      do
         extend ("Content-Type: text/plain")
         new_line
         new_line
         stdout.flush
      end


feature -- server push, multipart header

   content_multipart_x_mixed_replace (boundary: STRING) is
      require
         valid_boundary: boundary /= Void and then not boundary.is_empty
      do
         extend ("Content-Type: multipart/x-mixed-replace;boundary=")
         extend(boundary)
         my_boundary := boundary
         new_line
         new_line
         -- start first part immediately
         extend (my_boundary)
         new_line
         stdout.flush
      end

   content_next_part is
         -- write boundary so next part of multipart msg can be written
      require
         multipart: is_multipart_message
      do
         new_line
         extend (my_boundary)
         new_line
         stdout.flush
      end

   content_multipart_end is
         -- write boundary of multipart
      require
         multipart: is_multipart_message
      do
         new_line
         extend (my_boundary)
         extend ("--")
         new_line
         stdout.flush
      end

   is_multipart_message: BOOLEAN is
      do
         Result := my_boundary /= Void
      end


feature {NONE} -- private boundary state

   my_boundary: STRING


feature -- form input

   has_input: BOOLEAN is
         -- True if input passed to cgi program
      do
         Result := not request_method.is_empty
      end

   exist_value (key: STRING): BOOLEAN is
         -- returns True if key defined in input
      do
         locate_key (key)
         Result := exist_key
      end

   is_meta_char (c: CHARACTER): BOOLEAN is
         -- Return True if c is a commonly used meta characters
      do
         Result := meta_chars.has (c)
      end

   meta_chars: STRING is
      once
         -- check if this list is correct...
         Result := "`[]<>|&`\%%"
      end

   raw_value (key: STRING): STRING is
         -- returns value for key
         -- if key does not exist, the empty string is returned
      do
         locate_key (key)
         if exist_key then
            create Result.make_from_string (last_pair.value)
        else
            create Result.make (0)
         end
      end

   remove_meta_chars (s: STRING) is
         -- if s contains meta characters, they're removed
      local
         i: INTEGER
      do
         from
            i := 1
         until
            i > s.count
         loop
            if is_meta_char (s.item (i)) then
               s.remove (i)
            else
               i := i + 1
            end
         end
      end

   value (key: STRING): STRING is
         -- returns safe value for key, meta characters are removed
      do
         Result := raw_value (key)
         remove_meta_chars (Result)
      end


feature {NONE} -- cached key/value

   assert_key_value_pairs_created is
      -- make sure passed form contents (if any) is parsed
      do
         if cgi_data = Void then
            create cgi_data.make (0, -1)
            if has_input then
               fill_key_value_pairs
            end
         end
      end

   cgi_data: ARRAY[EPX_KEYVALUE]
         -- array with key-value pairs

   cgi_input: STRING
         -- raw input data

   exist_key: BOOLEAN is
      do
         Result := last_pair /= Void
      end

   fill_key_value_pairs is
         -- fill cgi_data, recognizes default and multipart encodings
      require
         key_value_pair_array_exists: cgi_data /= Void
         there_is_input: has_input
      local
         ctype: STRING
      do
         create ctype.make_from_string (content_type)
         ctype.to_lower
         if ctype.is_empty or else ctype.is_equal (standard_encoding) then
            fill_key_value_pairs_from_standard_encoding
         elseif ctype.substring_index (multipart_encoding, 1) = 1 then
            fill_key_value_pairs_from_multipart_encoding
         end -- ignore plaintext encoding
      end

   do_fill_key_value_pairs_from_mime (name: STRING; part: EPX_MIME_PART) is
      local
         subpart: EPX_MIME_PART
         subparts: ARRAY[EPX_MIME_PART]
         field: EPX_MIME_FIELD
         parameter: EPX_MIME_PARAMETER
         keyname,
         keyvalue: STRING
         i: INTEGER
         kv: EPX_KEYVALUE
         temporary_file: STDC_TEMPORARY_FILE
      do
         -- `part'.`body' contain the parameters
         if part.is_multipart then
            from
               subparts ?= part.body.value
               i := subparts.lower
            until
               i > subparts.upper
            loop
               subpart := subparts.item (i)

               -- we take care by checking if mime message is
               -- according to the specs. It should have a
               -- Content-Disposition field
               -- The name parameter can be absent in case of multiple
               -- uploaded files
               field := subpart.field ("Content-Disposition")
               if field /= Void then
                  parameter := field.parameter ("name")
                  if parameter /= Void or else name /= Void then
                     if parameter /= Void then
                        keyname := parameter.value
                     else
                        keyname := name
                     end

                     -- if multipart (multiple uploaded files), recurse
                     -- else singlepart with value
                     if subpart.is_multipart then
                        do_fill_key_value_pairs_from_mime (keyname, subpart)
                     else
                        parameter := field.parameter ("filename")
                        if parameter /= Void and then not parameter.value.is_empty then
                           keyvalue := remove_directory_part (parameter.value)
                           temporary_file := parameter.file
                        else
                           keyvalue ?= subpart.body.value
                        end
                        create kv.make (keyname, keyvalue)
                        if temporary_file /= Void then
                           kv.set_file (temporary_file)
                           temporary_file := Void
                        end
                        cgi_data.resize (cgi_data.lower, cgi_data.upper + 1)
                        cgi_data.put (kv, cgi_data.upper)
                     end
                  end
               end
               i := i + 1
            end
         else -- not recognized variant... perhaps accept?
         end
      end

   fill_key_value_pairs_from_multipart_encoding is
      local
         mime_header: STRING
         part: EPX_MIME_PART
         reader: EPX_FILE_READER
      do
         create reader.make (stdin)
         mime_header := "Content-Type: " + content_type
         create part.make_from_header (reader, mime_header)
         part.set_save_body (True)
         part.parse

         do_fill_key_value_pairs_from_mime (Void, part)
      end

   fill_key_value_pairs_from_standard_encoding is
      local
         mya, myb: ARRAY[STRING]
         s: STRING
         kv: EPX_KEYVALUE
         i:  INTEGER
         empty_field: BOOLEAN
         new_empty_string: STRING
      do
         set_cgi_input
         if cgi_input /= Void then
            mya := sh.split_on (cgi_input, '&')
            from
               i := mya.lower
            until
               i > mya.upper
            loop
               s := mya.item (i)
               myb := sh.split_on (s, '=')
               empty_field := myb.count = 1
               if empty_field then
                  create new_empty_string.make (0)
                  create kv.make (decode_string(myb.item(myb.lower)),
                                  new_empty_string)
               else
                  create kv.make (decode_string(myb.item(myb.lower)),
                                  decode_string(myb.item(myb.upper)))
               end
               cgi_data.resize (cgi_data.lower, cgi_data.upper + 1)
               cgi_data.put (kv, cgi_data.upper)
               i := i + 1
            end
         end
      end

   last_pair: EPX_KEYVALUE

   locate_key (key: STRING) is
         -- locate key in key/value pairs
      require
         valid_key: key /= Void and then not key.is_empty
      local
         i: INTEGER
         found: BOOLEAN
      do
         assert_key_value_pairs_created
         from
            last_pair := Void
            i := cgi_data.lower
         until
            found or else i > cgi_data.upper
         loop
            found := cgi_data.item (i).key.is_equal (key)
            if found then
               last_pair := cgi_data.item (i)
            else
               i := i + 1
            end
         end
      end

   set_cgi_input is
      local
         len: INTEGER
         i: INTEGER
      do
         if request_method.is_equal ("GET") then
            cgi_input := query_string
         elseif request_method.is_equal ("POST") then
            from
               len := content_length
               i := 0
               create cgi_input.make (64)
            until
               i = len
            loop
               stdin.read_character
               cgi_input.append_character (stdin.last_character)
               i := i + 1
            end
         end
      end


feature -- Microsoft Internet Explorer stuff

   remove_directory_part (filename: STRING): STRING is
         -- IE sends full path instead of just filename as everybody
         -- else does
      local
         path: STDC_PATH
      do
         create path.make_from_string (filename)
         path.parse (Void)
         path.set_directory (Void)
         Result := path
      end


feature -- decoding of encoded passed values, thanks Dustin Sallings

   decode_string(in: STRING): STRING is
      -- Decode an HTTP encoded string.
      local
         i: INTEGER
         c: CHARACTER
      do
         create Result.make(64)
         from
            i := 1
         until
            i > in.count
         loop
            c := in.item (i)
            if c = '+' then
               Result.append_character (' ')
            elseif c = '%%' then
               i := i + 1
               Result.append_character (from_hex(in.substring(i, i + 1)))
               i := i + 1
            else
               Result.append_character (c)
            end
            i := i + 1
         end
      end

   from_hex(in: STRING): CHARACTER is
      -- Take a two digit hex string, and convert it to a hex character.
      require
         has_two_digits: in.count = 2
         -- in.item(1).is_hexadecimal_digit;
         -- in.item(2).is_hexadecimal_digit;
      local
         mya, myb: INTEGER;
         map, tmp: STRING;
      do
         tmp := clone (in)
         tmp.to_upper
         map := "0123456789ABCDEF"
         mya := map.index_of(tmp.item(1), 1) - 1
         myb := map.index_of(tmp.item(2), 1) - 1
         mya := mya * 16 + myb
         Result := ch.from_integer (mya)
      end


end -- class EPX_CGI
