indexing

   description: "Class that covers POSIX child process."

   author: "Berend de Boer"
   date: "$Date: 2001/12/27 $"
   revision: "$Revision: #1 $"


deferred class

   POSIX_CHILD_PROCESS


inherit

   POSIX_PROCESS

   ABSTRACT_CHILD_PROCESS


feature -- child's pid

   pid: INTEGER is
      do
         Result := my_pid
      end

   is_pid_valid: BOOLEAN is
         -- return True if this object refers to a child process, so
         -- it has an id
      do
         Result := my_pid > 0
      end


feature -- actions that parent may execute

   wait_for (suspend: BOOLEAN) is
         -- wait for this process to terminate. If `suspend' then we
         -- wait until the information about this process is available,
         -- else we return immediately.
         -- If suspend is False, check the running property to see
         -- if this child is really terminated.
      local
         wait_pid: INTEGER
         stat_loc: ARRAY[INTEGER]
         options: INTEGER
      do
         create stat_loc.make (0, 0)
         options := 0
         if not suspend then
            options := flip_bits (options, WNOHANG, True)
         end
         wait_pid :=
            posix_waitpid (my_pid,
                           ah.integer_array_to_pointer (stat_loc),
                           options)
         if wait_pid = -1 then
            raise_posix_error
         end
         running := wait_pid <= 0
         if not running then
            my_pid := 0
         end
         termination_info := stat_loc.item(0)
      end



feature {POSIX_CURRENT_PROCESS}

   set_pid (a_pid: INTEGER) is
         -- initialize pid as seen by parent, just after a fork
      do
         my_pid := a_pid
      end


feature {NONE} -- private state

   my_pid: INTEGER

   termination_info: INTEGER
         -- also known as statloc

end
