indexing

   description: "Abstract daemon class."
   
   usage: "Inherit from this class, implement `execute' and detach."

   author: "Berend de Boer"
   date: "$Date: 2001/12/27 $"
   revision: "$Revision: #1 $"


deferred class 

   POSIX_DAEMON
   

inherit

   POSIX_FORK_ROOT
      redefine
         after_fork
      end

   CAPI_SIGNAL

   PAPI_UNISTD

   PAPI_STAT


feature -- daemon specific actions   
   
   detach is
         -- detach from command-line, not very useful if you want to
         -- spawn multiple daemons, but you can always pass daemons to
         -- the fork routine yourself.
      do
         fork (Current)
      end
   
   after_fork is
         -- Code thanks to W. Richard Stevens
      local
         pr: POINTER
         r: INTEGER
         i: INTEGER
      do
         -- become session leader
         safe_call (posix_setsid)
         
         -- ignore SIGHUP
         pr := posix_signal (SIGHUP, SIG_IGN)
         if pr = SIG_ERR then
            raise_posix_error
         end
         
         -- guarantee we cannot acquire a controlling terminal by
         -- forking again
         if posix_fork /= 0 then
            -- let first child terminate
            exit (EXIT_SUCCESS)
         end
         
         -- change working directory to root, so volume can be unmounted
         r := posix_chdir (sh.string_to_pointer ("/"))
         
         -- clear our file mode creation mask
         r := posix_umask (0)
         
         -- clear first 64 (open) file descriptors here
         from
            i := 0
         until
            i = 64
         loop
            r := posix_close (i)
            i := i + 1
         end
         
         -- perhaps open stdin, stdout and stderr to /dev/null here?
      end

   
end -- class POSIX_DAEMON
