indexing

	description: "Test EPX exec process class."

  author: "Berend de Boer"
	date: "$Date: 2001/12/27 $"
	revision: "$Revision: #1 $"

class 

   TEST_EPX_EXEC_PROCESS

inherit
   
   EPX_CURRENT_PROCESS


creation
   
   make

   
feature

   make is
      local
         ls,
         more: EPX_EXEC_PROCESS
         i: INTEGER
      do
         print ("Begin test exec process.%N")         
         
         print ("My pid: ")
         print (pid)
         print ("%N")
         
         -- test exec process itself by just doing an ls
         print ("Test plain ls without capturing.%N")         
         create ls.make ("ls", Void)
         ls.execute
         ls.wait_for (True)

         -- execute ls and print output
         -- note that this reads blocks, not lines...
         print ("Test output capturing.%N")         
         create ls.make_capture_output ("ls", Void)
         ls.execute
         print ("ls pid: ")
         print (ls.pid)
         print ("%N")
         from
            ls.fd_stdout.read_string (512)
         until 
            ls.fd_stdout.eof 
         loop
            print (ls.fd_stdout.last_string)
            ls.fd_stdout.read_string (512)
         end
         
         -- close stdout (needed?) and wait
         ls.fd_stdout.close
         ls.wait_for (True)
         
         -- test input capturing
         -- more.com is very unreliable...
         print ("Test input capturing.%N")         
         create more.make_capture_input ("less", Void)
         -- create more.make_capture_input ("more", Void)
         more.execute
         print ("more pid: ")
         print (more.pid)
         print ("%N")
         sleep (3)
         from
            i := 0
         until 
            i = 25
         loop
            more.fd_stdin.put_string (i.out)
            more.fd_stdin.put_string ("%N")
            i := i + 1
         end
         
         -- send EOF to more so it will exit
         more.fd_stdin.close

         -- wait for more to terminate
         more.wait_for (True)
         
         -- execute ls and pipe output to more
         print ("Test output and input capturing.%N")         
         create ls.make_capture_output ("ls", Void)
         create more.make_capture_input ("less", Void)
         ls.execute
         print ("ls pid: ")
         print (ls.pid)
         print ("%N")
         more.execute
         print ("more pid: ")
         print (more.pid)
         print ("%N")
         from
            ls.fd_stdout.read_string (512)
         until 
            ls.fd_stdout.eof or else more.fd_stdin.eof
         loop
            --print (ls.fd_stdout.last_string)
            more.fd_stdin.put_string (ls.fd_stdout.last_string)
            ls.fd_stdout.read_string (512)
         end

         -- close captured io for both processes
         ls.fd_stdout.close
         more.fd_stdin.close

         -- wait for both (!) processes
         ls.wait_for (True)
         more.wait_for (True)

         print ("Test exit code.%N")
         ls.set_arguments (<<"QQ*">>)
         ls.execute
         ls.wait_for (True)
         print ("Exit code: ")
         print (ls.exit_code)
         print ("%N")

         print ("Test wait_for.%N")
         ls.set_arguments (<<"QQ*">>)
         ls.execute
         ls.wait_for (False)
         if not ls.is_terminated then
            print ("Status not yet available.%N")
            ls.wait_for (True)
         end
         print ("Exit code: ")
         print (ls.exit_code)
         print ("%N")

         test_fail

         print ("End test exec process.%N")         
      end


   test_fail is
      local
         rescued: BOOLEAN
         exec: EPX_EXEC_PROCESS
      do
         if not rescued then
            create exec.make ("/usr/local/bin/nonexistent", Void)
            exec.execute
            exec.wait_for (True)
            if exec.exit_code = EXIT_SUCCESS then
               print ("!! not existing program not detected.")
            end
         end
      rescue
         rescued := True
         retry
      end
   
end -- class TEST_EPX_EXEC_PROCESS
